"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
var _react = require("react");
var _react2 = require("@testing-library/react");
var _vitest = require("vitest");
var _userEvent = _interopRequireDefault(require("@testing-library/user-event"));
require("@testing-library/jest-dom");
var _runAxeCheck = require("@instructure/ui-axe-check/lib/runAxeCheck.js");
var _index = require("../index");
var _jsxRuntime = require("@emotion/react/jsx-runtime");
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015 - present Instructure, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

const TEST_VALUE = 'test-value';
const TEST_NAME = 'test-name';
const TEST_LABEL = 'test-label';
const initProps = {
  label: TEST_LABEL,
  defaultChecked: true,
  value: TEST_VALUE,
  name: TEST_NAME
};
const renderCheckbox = props => {
  const allProps = {
    ...initProps,
    ...props
  };
  return (0, _react2.render)((0, _jsxRuntime.jsx)(_index.Checkbox, {
    ...allProps
  }));
};
describe('<Checkbox />', () => {
  let consoleWarningMock;
  let consoleErrorMock;
  beforeEach(() => {
    // Mocking console to prevent test output pollution
    consoleWarningMock = _vitest.vi.spyOn(console, 'warn').mockImplementation(() => {});
    consoleErrorMock = _vitest.vi.spyOn(console, 'error').mockImplementation(() => {});
  });
  afterEach(() => {
    consoleWarningMock.mockRestore();
    consoleErrorMock.mockRestore();
  });
  it('renders an input with type "checkbox"', () => {
    renderCheckbox();
    const inputElem = _react2.screen.getByRole('checkbox');
    expect(inputElem).toBeInTheDocument();
    expect(inputElem.tagName).toBe('INPUT');
    expect(inputElem).toHaveAttribute('type', 'checkbox');
  });
  it('`simple` variant only displays a checkmark when checked', async () => {
    const _renderCheckbox = renderCheckbox({
        variant: 'simple',
        defaultChecked: false
      }),
      container = _renderCheckbox.container;
    const checkboxElement = container.querySelector('input[type="checkbox"]');
    const svgElement = container.querySelector('svg');
    expect(svgElement).not.toBeInTheDocument();
    _userEvent.default.click(checkboxElement);
    await (0, _react2.waitFor)(() => {
      const svgElementAfterClick = container.querySelector('svg');
      expect(svgElementAfterClick).toBeInTheDocument();
    });
  });
  it('`simple` variant supports indeterminate/mixed state', () => {
    renderCheckbox({
      variant: 'simple',
      indeterminate: true
    });
    const inputElem = _react2.screen.getByRole('checkbox');
    expect(inputElem).toBeInTheDocument();
    expect(inputElem).toHaveAttribute('aria-checked', 'mixed');
  });
  describe('events', () => {
    it('when clicked, fires onClick and onChange events', async () => {
      const onClick = _vitest.vi.fn();
      const onChange = _vitest.vi.fn();
      renderCheckbox({
        onClick,
        onChange
      });
      const checkboxElement = _react2.screen.getByRole('checkbox');
      _userEvent.default.click(checkboxElement);
      await (0, _react2.waitFor)(() => {
        expect(onClick).toHaveBeenCalled();
        expect(onChange).toHaveBeenCalled();
      });
    });
    it('when clicked, does not call onClick or onChange when disabled', async () => {
      const onClick = _vitest.vi.fn();
      const onChange = _vitest.vi.fn();
      renderCheckbox({
        onClick,
        onChange,
        disabled: true
      });
      const checkboxElement = _react2.screen.getByRole('checkbox');
      _react2.fireEvent.click(checkboxElement);
      await (0, _react2.waitFor)(() => {
        expect(onClick).not.toHaveBeenCalled();
        expect(onChange).not.toHaveBeenCalled();
        expect(checkboxElement).toBeDisabled();
      });
    });
    it('when clicked, does not call onClick or onChange when readOnly', async () => {
      const onClick = _vitest.vi.fn();
      const onChange = _vitest.vi.fn();
      renderCheckbox({
        onClick,
        onChange,
        readOnly: true
      });
      const checkboxElement = _react2.screen.getByRole('checkbox');
      _react2.fireEvent.click(checkboxElement);
      await (0, _react2.waitFor)(() => {
        expect(onClick).not.toHaveBeenCalled();
        expect(onChange).not.toHaveBeenCalled();
      });
    });
    it('calls onChange when enter key is pressed', async () => {
      const onChange = _vitest.vi.fn();
      renderCheckbox({
        onChange
      });
      const checkboxElement = _react2.screen.getByRole('checkbox');
      _userEvent.default.type(checkboxElement, '{enter}');
      await (0, _react2.waitFor)(() => {
        expect(onChange).toHaveBeenCalled();
      });
    });
    it('responds to onBlur event', async () => {
      const onBlur = _vitest.vi.fn();
      renderCheckbox({
        onBlur
      });
      _userEvent.default.tab();
      _userEvent.default.tab();
      await (0, _react2.waitFor)(() => {
        expect(onBlur).toHaveBeenCalled();
      });
    });
    it('responds to onFocus event', async () => {
      const onFocus = _vitest.vi.fn();
      renderCheckbox({
        onFocus
      });
      _userEvent.default.tab();
      await (0, _react2.waitFor)(() => {
        expect(onFocus).toHaveBeenCalled();
      });
    });
    it('focuses with the focus helper', () => {
      var _checkboxRef$current;
      const checkboxRef = /*#__PURE__*/(0, _react.createRef)();
      (0, _react2.render)((0, _jsxRuntime.jsx)(_index.Checkbox, {
        ref: checkboxRef,
        ...initProps
      }));
      const checkboxElement = _react2.screen.getByRole('checkbox');
      expect(checkboxElement).not.toHaveFocus();
      (_checkboxRef$current = checkboxRef.current) === null || _checkboxRef$current === void 0 ? void 0 : _checkboxRef$current.focus();
      expect(checkboxElement).toHaveFocus();
    });
    it('calls onMouseOver', async () => {
      const onMouseOver = _vitest.vi.fn();
      renderCheckbox({
        onMouseOver
      });
      const checkboxElement = _react2.screen.getByRole('checkbox');
      _userEvent.default.hover(checkboxElement);
      await (0, _react2.waitFor)(() => {
        expect(onMouseOver).toHaveBeenCalled();
      });
    });
    it('calls onMouseOut', async () => {
      const onMouseOut = _vitest.vi.fn();
      renderCheckbox({
        onMouseOut
      });
      const checkboxElement = _react2.screen.getByRole('checkbox');
      _userEvent.default.hover(checkboxElement);
      _userEvent.default.unhover(checkboxElement);
      await (0, _react2.waitFor)(() => {
        expect(onMouseOut).toHaveBeenCalled();
      });
    });
  });
  describe('for a11y', () => {
    it('`simple` variant should meet standards', async () => {
      const _renderCheckbox2 = renderCheckbox({
          variant: 'simple'
        }),
        container = _renderCheckbox2.container;
      const axeCheck = await (0, _runAxeCheck.runAxeCheck)(container);
      expect(axeCheck).toBe(true);
    });
    it('`toggle` variant should meet standards', async () => {
      const _renderCheckbox3 = renderCheckbox({
          variant: 'toggle'
        }),
        container = _renderCheckbox3.container;
      const axeCheck = await (0, _runAxeCheck.runAxeCheck)(container);
      expect(axeCheck).toBe(true);
    });
  });
});